<?php
require_once("lib-action/action.php");
require_once("UI/Page.php");

class BatchFilesSettingsEditPage extends UIPage
{
	public function createControls()
	{
		$settings_panel = UIFactory()->InputPanel(blmsg("TRANS_SETTINGS"));
		$process_days_count = UIFactory()->InputString("batchFileManager_ProcessDaysCount", blmsg("TRANS_BATCH_FILES_PROCESS_DAYS_COUNT"), true);
		$process_days_count->value = Settings()->batchFileManager_ProcessDaysCount;
		$settings_panel->addInputControl($process_days_count);
		$process_transactions_count = UIFactory()->InputString("batchFileManager_ProcessTransactionsCount", blmsg("TRANS_BATCH_FILES_PROCESS_TRANSACTION_COUNT"), true);
		$process_transactions_count->value = Settings()->batchFileManager_ProcessTransactionsCount;
		$settings_panel->addInputControl($process_transactions_count);

		$responsible_admin = UIFactory()->InputDropDown( "batchFileManager_ResponsibleAdmin",blmsg("TRANS_RESPONSIBLE_ADMIN") );
		$responsible_admin->value = Settings()->batchFileManager_ResponsibleAdmin;
		foreach ( Factory()->AdminList() as $admin )
			$responsible_admin->addOption( $admin->id, "{$admin->admin_first_name} {$admin->admin_last_name} ({$admin->admin_username})");

		$settings_panel->addInputControl($responsible_admin);
		$this->addInputPanel($settings_panel);
	}

	public function processInput()
	{
		require_once("Billing_Checker.php");
		$days = $this->getValue("batchFileManager_ProcessDaysCount");
		$transactions = $this->getValue("batchFileManager_ProcessTransactionsCount");
		$admin = $this->getValue("batchFileManager_ResponsibleAdmin");
		$errors = array();
		if (0 == Billing_Checker::checkRegexp(Billing_Checker::P_UINT_NOT_NULL, $days))
			$errors[] = blmsg("TRANS_ERROR_WRONG_PARAMETER_VALUE", array("PARAMETER" => blmsg("TRANS_BATCH_FILES_PROCESS_DAYS_COUNT")));

		if (0 == Billing_Checker::checkRegexp(Billing_Checker::P_UINT_NOT_NULL, $transactions))
			$errors[] = blmsg("TRANS_ERROR_WRONG_PARAMETER_VALUE", array("PARAMETER" => blmsg("TRANS_BATCH_FILES_PROCESS_TRANSACTION_COUNT")));

		if (0 == Billing_Checker::checkRegexp(Billing_Checker::P_UINT_NOT_NULL, $admin))
			$errors[] = blmsg("TRANS_ERROR_WRONG_PARAMETER_VALUE", array("PARAMETER" => blmsg("TRANS_RESPONSIBLE_ADMIN")));

		if ( !empty($errors) )
		{
			$this->addErrorMessage(UIFactory()->ErrorMessage($errors));
			return false;
		}

		Settings()->batchFileManager_ProcessDaysCount = $days;
		Settings()->batchFileManager_ProcessTransactionsCount = $transactions;
		Settings()->batchFileManager_ResponsibleAdmin = $admin;
		$this->addInfoMessage(UIFactory()->InfoMessage(blmsg("TRANS_UPDATE_SUCCESSFUL")));
		return false;
	}

	protected function getSubmitButtons()
	{
		return array(UIFactory()->SubmitButton("ok", blmsg("TRANS_SAVE"), UIPageController::CMD_OK));
	}
}

class EditBatchFilesSettings extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$controller  = UIFactory()->PageController();
		$controller->addPage($edit_page = new BatchFilesSettingsEditPage($controller));

		require_once("UI/Tabs/UIBatchFilesScreenTabs.php");
		$batch_files_tabs = new UIBatchFilesScreenTabs();
		$batch_files_tabs->setCurrent("settings");
		$edit_page->setScreenTabs($batch_files_tabs);
		$edit_page->title = blmsg("TRANS_SETTINGS");
		$edit_page->parent_title = blmsg("TRANS_BATCH_FILES_MANAGER");
		$edit_page->parent_url = action_url("batchfiles", "ShowBatchFiles");
		$controller->processRequest();
		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}