<?php

require_once(dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php");
require_once("lib-action/action.php");

class EditBillingAccount extends MB_Action
{
	public $params = array("clientID", "clientContactID", "billingAccountID");

	public function execute()
	{
		global $CONF;
		parent::execute();

		$clientID = getParam('clientID');
		$clientContactID = getParam('clientContactID');
		$billingAccountID = getParam('billingAccountID');
		if (!$clientContactID && $billingAccountID){
			$accountInfo = oneByID("billingAccounts", $billingAccountID);
			$clientContactID = $accountInfo["clientContactID"];
		}
		if (!$clientID && $clientContactID){
			$contactInfo = oneByID("clientContacts", $clientContactID);
			$clientID = $contactInfo["clientID"];
		}

		if(!$billingAccountID) {
			$this->addError(TRANS_ACTIONERROR, "No billing account ID given.");
		} else if(!$clientContactID) {
			$this->addError(TRANS_ACTIONERROR, "No client contact ID given.");
		} else if(!$clientID) {
			$this->addError(TRANS_ACTIONERROR, "No client ID given.");
		} else {

			require_once("object_managers/CryptManager.php");
			$decParams = array();
			if ( 1 == bgpc("showDecryptedCC") ) {
				switch ( CryptManager::getEncryptionMode() ) {
					case CryptManager::MODE_LEK:
						$decParams = array("getDecryptedCC" => 1, "lekPin" => bgpc("lekPin"));
						break;
					case CryptManager::MODE_PASSPHRASE:
						$decParams = array("getDecryptedCC" => 1, "passphrase" => bgpc("passphrase"));
				}
			}
			$query = $this->getMBAPIQuery("GetClients", "", array("getContactData"=>"4", "getBillingData"=>"4", "clientContactID" =>$clientContactID) + $decParams);
			$data = dispatchMBAPI($query, 1);

			$results =& $data["mbapi"][0]["results"][0];

			if ($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors())
			{
				require_once("object_managers/SecurityManager.php");
				SecurityManager::pushCurrentActionIntoLog(
					SecurityManager::USER_TYPE_CLIENT,
					$clientID,
					Factory()->Client($clientID)->PrimaryContact->contact_username,
					"TRANS_ACTION_LOG_REASON_SHOW_BILLING_ACCOUNT_DETAILS",
					blmsg(
						(1 == bgpc("showDecryptedCC") ? "TRANS_ACTION_LOG_COMMENT_CLIENT_BILLING_ACCOUNT_VIEW_DECRYPTED_DATA" : "TRANS_ACTION_LOG_COMMENT_CLIENT_BILLING_ACCOUNT"),
						array("ACCOUNT_NAME" => Factory()->ClientBillingAccount($billingAccountID)->getAccountNameForList(), "ACCOUNT_ID" => $billingAccountID, "CLIENT_ID" => $clientID),
						SecurityManager::getActionLogLocale()
					)
				);

				$this->smartyVars["client"] = correctArray($results["clients"][0]["client"][0]);
				$this->smartyVars["contact"] = correctArray($results["clients"][0]["client"][0]["contacts"][0]["contact"][0]);
				$this->smartyVars["billingAccount"] = collapseArrayElements($results["clients"][0]["client"][0]["billingAccounts"][0]["billingAccount"][0]);
				$this->smartyVars["billingAccountTypes"] = $CONF["types"]["billingAccount"];
				$this->smartyVars["bankAccountTypes"] = $CONF["types"]["bankAccount"];
				$this->smartyVars["cardTypes"] = $CONF["types"]["card"];
				$this->smartyVars["cardExtraTypes"] = $CONF["types"]["cardExtraIDs"];

				// HSPC Bank Transfer
				if (getAPITypeID("billingAccount", "hspcbanktransfer") == $this->smartyVars["billingAccount"]["billingAccountType"])
				{
					require_once("ModuleTool.php");
					$client_contact_id = NULL;

					$hspc_pp_plugin = ModuleTool::getModuleInstance("gateway", "HSPCBankTransfer", array(
						"gateway_id" => $this->smartyVars["billingAccount"]["paymentGatewayID"],
						"billing_account_id" => $this->smartyVars["billingAccount"]["billingAccountID"],
					));
					$form = $hspc_pp_plugin->getPaymentMethodForm("edit");
					$this->smartyVars["hspc_payment_method_form"] = $form["form_content"];
				}

				$this->smartyVars["encryptionMode"] = CryptManager::getEncryptionMode();

				// Subscriptions
				$this->smartyVars["selectedPackages"] = grabHTMLOpsFromArray(grabResultsFromMBAPIArray(array(
					"mbapi" => array(
						"command" => "GetBillingAccountShares",
						"params" => array(
							"billingAccountID" => $billingAccountID,
							"getPackageData" => 1,
							"billingAccountShareValueGreaterThan" => 0
						),
					),
				)), "packageID", "baMakePackageName");
				$this->smartyVars["selectablePackages"] = grabHTMLOpsFromArray(grabResultsFromMBAPIArray(array(
					"mbapi" => array(
						"command" => "GetPackages",
						"params" => array(
							"clientID" => $clientID,
						),
					),
				)), "packageID", "baMakePackageName", array_keys($this->smartyVars["selectedPackages"]));

				$defaults = $this->smartyVars["billingAccount"];
				if (!is_null($billingAccountType = bgpc('billingAccountType')))
				{
					$defaults = array('billingAccountType' => $billingAccountType);
				}

				$this->mergeFormState($defaults);
				loadConfigs("payment", "paymentQueue");
				$this->smartyVars["paymentQueueSettings"] =& $CONF["payment"]["paymentQueue"];
				$this->addTemplate("clients/EditBillingAccount", $this->smartyVars);
			}
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}

function baMakePackageName($row)
{
	return "(".TRANS_ID.": ".$row["packageID"].") ".$row["packageName"];
}
