<?php

require_once(dirname(__FILE__)."/../../../../lib-tk/shield.php");
require_once("lib-action/genericactions.php");

class EditBillingAccountResponse extends EditResponseAction
{
	public function __construct()
	{
		if(!$billingAccount = oneOf("billingAccounts", array("billingAccountID" => (int) $_REQUEST["billingAccountID"]))) {
			$this->addError(MBAPI_ACTIONERROR, "No valid match.");
			return false;
		}

		$this->d = new DataSource("billingAccounts");
		$description = $this->d->getDescription();
		parent::__construct();
		unset($description['columns']['clientContactID']);

		$billingAccountType = (int) $_REQUEST["billingAccountType"];
		if($billingAccountType == getAPITypeID("billingAccount", "manual") || $billingAccountType == getAPITypeID("billingAccount", "secondary")) {
			unset($description["columns"]['billingBankType']);
			unset($description["columns"]['billingBankName']);
			unset($description["columns"]['billingBankState']);
			unset($description["columns"]['billingBankABACode']);
			unset($description["columns"]['billingBankAccountNum']);
			unset($description["columns"]['billingBankLicenseNum']);
			unset($description["columns"]['billingBankLicenseState']);
			unset($description["columns"]['billingBankDob']);

			unset($description["columns"]['billingCardNameOnCard']);
			unset($description["columns"]['billingCardType']);
			unset($description["columns"]['billingCardNum']);
			unset($description["columns"]['billingCardExpMonth']);
			unset($description["columns"]['billingCardExpYear']);
			unset($description["columns"]['billingCardIssueNum']);
			unset($description["columns"]['billingCardStartDate']);
		} else if($billingAccountType == getAPITypeID("billingAccount", "card")) {
			unset($description["columns"]['billingBankType']);
			unset($description["columns"]['billingBankName']);
			unset($description["columns"]['billingBankState']);
			unset($description["columns"]['billingBankABACode']);
			unset($description["columns"]['billingBankAccountNum']);
			unset($description["columns"]['billingBankLicenseNum']);
			unset($description["columns"]['billingBankLicenseState']);
			unset($description["columns"]['billingBankDob']);

			$billingCardType = (int) $_REQUEST["billingCardType"];
			$cardExtraTypes = getAPITypes("cardExtraIDs");
			if(!in_array($billingCardType, $cardExtraTypes)) {
				unset($description["columns"]['billingCardIssueNum']);
				unset($description["columns"]['billingCardStartDate']);
			}
			if (!$_REQUEST["billingCardNum"] || $_REQUEST["billingCardNum"]==""){
				unset($description["columns"]['billingCardNum']);
			}
		} else {
			unset($description["columns"]['billingCardNameOnCard']);
			unset($description["columns"]['billingCardType']);
			unset($description["columns"]['billingCardNum']);
			unset($description["columns"]['billingCardExpMonth']);
			unset($description["columns"]['billingCardExpYear']);
			unset($description["columns"]['billingCardIssueNum']);
			unset($description["columns"]['billingCardStartDate']);
		}
		$this->d->setDescription($description);
		switch ($billingAccountType)
		{
			case getAPITypeID("billingAccount", "card"):
				$validation_fields = array(
					"billingCardNameOnCard" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
					"billingCardExpMonth" => INPUT_RESTRICTION_MONTH|INPUT_RESTRICTION_REQUIRED,
					"billingCardExpYear" => INPUT_RESTRICTION_YEAR|INPUT_RESTRICTION_REQUIRED,
					"billingCardNum" => array(
						INPUT_RESTRICTION_INT|INPUT_RESTRICTION_MAXLEN|INPUT_RESTRICTION_MINLEN|INPUT_RESTRICTION_CARD,
						array(
							'maxLength' => 19,
							'minLength' => 13,
							'allowedCardTypes' => array(getAPITypeName("card", bgpc('billingCardType'))),
						),
					),
                );
				break;
			case getAPITypeID("billingAccount", "echeck"):
			case getAPITypeID("billingAccount", "bank"):
			case getAPITypeID("billingAccount", "wire"):
				$validation_fields = array(
					"billingBankType" => INPUT_RESTRICTION_UNSIGNED_INT,
					"billingBankName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
					"billingBankState" => INPUT_RESTRICTION_STRING,
					"billingBankAbaCode" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
					"billingBankAccountNum" => INPUT_RESTRICTION_UNSIGNED_INT,
					"billingBankLicenseNum" => INPUT_RESTRICTION_STRING,
					"billingBankLicenseState" => INPUT_RESTRICTION_STRING,
                );
				break;
		}
		$this->setReferralActionID(getActionID("EditBillingAccount"));
		$this->setReferralActionFile("clients.php");
		$args = array("billingAccountID" => bgpc("billingAccountID"), "clientID" => bgpc("clientID"), "clientContactID" => bgpc("clientContactID"), "billingAccountType" => $billingAccountType);
		$this->validateMultiInputs( $validation_fields, array(), array(), 1, $args );
	}

	public function editResponseParamHandler()
	{
		if($_REQUEST["encKey"]) {
			$this->d->mergeParams(array("encKey" => $_REQUEST["encKey"]));
		}
		return true;
	}

	public function editResponseHandler()
	{
		global $CONF;
		// valid match assumed due to earlier check
		$billingAccount = Factory()->ClientBillingAccount(bgpc("billingAccountID"));
		$clientID = $billingAccount->ClientContact->Client->id;

		if ((int)$_REQUEST['defaultBillingAccount']){
			updateForKey(":clientsGeneric", $clientID, array(
				"defaultBillingAccountID" => $billingAccount->id,
			));
		}

		loadConfigs("payment", "paymentQueue");

		// HSPC Bank Transfer Account
		if (getAPITypeID("billingAccount", "hspcbanktransfer") == $_REQUEST["billingAccountType"])
		{
			require_once("ModuleTool.php");

			$hspc_result = ModuleTool::getModuleInstance("gateway", "HSPCBankTransfer", array(
				"billing_account_id" => $billingAccount->id,
			))->collectPaymentMethodForm(true); // Public Data only

			if (! is_null($hspc_result["collected_data"]->user_error))
			{
				$this->addError(blmsg("TRANS_ERROR_MESSAGES"), blmsg("TRANS_HSPC_BANK_TRANSFER_ACCOUNT_ERROR", array("error" => $hspc_result["collected_data"]->user_error)));
				return false;
			}
			else
			{
				$billingAccount->ClientBillingAccountParams["hspc_public_data"]->value
					= serialize($hspc_result["collected_data"]->plugin_data->public_data);

				require_once("object_managers/ClientBillingAccountManager.php");
				ClientBillingAccountManager::update($billingAccount);
			}
		}

		if (!(int)$CONF["payment"]["paymentQueue"]["enable_advanced_share_editing"]){
			// we need to remove shares for other billing accounts
			$shares = grabResultsFromMBAPIArray(array(
				"mbapi" => array(
					"command" => "GetBillingAccountShares",
					"params" => array(
						"billingAccountID" => $billingAccount->id,
					),
				),
			));

			$existingShares = array();
			foreach ($shares as $share){
				$existingShares[$share["packageID"]] = $share;
			}

			$newShares = array();
			$updateShares = array();
			foreach ((array)$_REQUEST["packages"] as $packageID){
				if ($existingShares[$packageID]){
					$updateShares[$existingShares[$packageID]["billingAccountShareID"]] = 1;
					unset($existingShares[$packageID]);
				} else {
					$newShares[$packageID] = 1;
				}
			}

			if ((int)$_REQUEST["packages"][0]){
				$oldShares = arrayFor(":clientBillingAccountShares", array(
					"packageID" => $_REQUEST["packages"],
				));
				foreach ((array)$oldShares as $os){
					updateForKey(":clientBillingAccountShares", $os["billingAccountShareID"], array(
						"billingAccountShareValue" => 0,
					));
				}
			}

			foreach ($newShares as $packageID=>$shareValue){
				dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetBillingAccountShare",
						"subCommand" => "insert",
						"params" => array(
							"billingAccountID" => $billingAccount->id,
							"packageID" => $packageID,
							"billingAccountFallbackGroup" => 1,
							"billingAccountShareGroup" => 0,
							"billingAccountShareValue" => $shareValue,
						),
					),
				)));
			}

			foreach ($updateShares as $shareID=>$shareValue){
				dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetBillingAccountShare",
						"subCommand" => "update",
						"params" => array(
							"billingAccountShareID" => $shareID,
							"billingAccountShareValue" => $shareValue,
						),
					),
				)));
			}

			foreach ($existingShares as $packageID=>$share){
				$shareID = $share["billingAccountShareID"];
				$shareValue = 0;
				dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetBillingAccountShare",
						"subCommand" => "update",
						"params" => array(
							"billingAccountShareID" => $shareID,
							"billingAccountShareValue" => $shareValue,
						),
					),
				)));
			}
		} else {
			// we need to preserve shares for other billing accounts
			$shares = grabResultsFromMBAPIArray(array(
				"mbapi" => array(
					"command" => "GetBillingAccountShares",
					"params" => array(
						"billingAccountID" => $billingAccount->id,
					),
				),
			));

			$existingShares = array();
			foreach ($shares as $share){
				$existingShares[$share["packageID"]] = $share;
			}

			$newShares = array();
			$updateShares = array();
			foreach ((array)$_REQUEST["packages"] as $packageID){
				if ($existingShares[$packageID]){
					$updateShares[$existingShares[$packageID]["billingAccountShareID"]] = 1;
					unset($existingShares[$packageID]);
				} else {
					$newShares[$packageID] = 1;
				}
			}

			foreach ($newShares as $packageID=>$shareValue){
				dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetBillingAccountShare",
						"subCommand" => "insert",
						"params" => array(
							"billingAccountID" => $billingAccount->id,
							"packageID" => $packageID,
							"billingAccountFallbackGroup" => 1,
							"billingAccountShareGroup" => 0,
							"billingAccountShareValue" => $shareValue,
						),
					),
				)));
			}

			foreach ($updateShares as $shareID=>$shareValue){
				dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetBillingAccountShare",
						"subCommand" => "update",
						"params" => array(
							"billingAccountShareID" => $shareID,
							"billingAccountShareValue" => $shareValue,
						),
					),
				)));
			}

			foreach ($existingShares as $packageID=>$share){
				$shareID = $share["billingAccountShareID"];
				$shareValue = 0;
				dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetBillingAccountShare",
						"subCommand" => "update",
						"params" => array(
							"billingAccountShareID" => $shareID,
							"billingAccountShareValue" => $shareValue,
						),
					),
				)));
			}

		}

		//////////////////////////
		return true;
	}

	public function overrideValidateColumnParamType($column, $columnType, $mode, $paramType, $isRequired)
	{
		switch($column) {
			case "billingCardNum":
				if(empty($_REQUEST["billingCardNum"])) {
					$columnType = "";
					break;
				}
				$paramType |= INPUT_RESTRICTION_MAXLEN|INPUT_RESTRICTION_MINLEN|INPUT_RESTRICTION_CARD;
				$args = array(
					'maxLength' => 19,
					'minLength' => 13,
					'allowedCardTypes' => array(getAPITypeName("card", $_REQUEST['billingCardType'])),
				);
				break;
			case "billingBankAccountNum":
				if(empty($_REQUEST["billingBankAccountNum"])) {
					$columnType = "";
					break;
				}
				break;
			default:
				$args = array();
				break;
		}

		return array($paramType, $isRequired, $args);
	}

	public function redirect()
	{
		if(!$this->goBack()) {
			$billingAccounts = new DataSource("billingAccounts", array("billingAccountID" => (int) $_REQUEST["billingAccountID"]));
			$contacts = new DataSource("contacts");
			$clients = new DataSource("clients");

			$clientID = $contacts->getIDByID($clients, $contactID = $billingAccounts->getIDFor(new DataSource("contacts")));

			redirect("clients.php", getActionID("ShowBillingAccounts"), array("clientID" => $clientID, "contact" => $contactID));
		}
	}
}
