<?php
// +------------------------------------------------------------------------------+
// | MBAPI [TM] .:. Client Billing Engine                                         |
// +------------------------------------------------------------------------------+
// | Copyright (c) 2001-2006 ModernGigabyte, LLC                                  |
// +------------------------------------------------------------------------------+
// | This source file is subject to the ModernGigabyte End User License Agreement |
// | (EULA), that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at http://www.mbapi.com/eula.php     |
// |                                                                              |
// | If you did not receive a copy of the MBAPI license and are unable to obtain  |
// | it through the world-wide-web, please send a note to license@mbapi.com so we |
// | can email you a copy immediately.                                            |
// +------------------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>                       |
// | Support: http://www.modernsupport.com/                                       |
// +------------------------------------------------------------------------------+
// | ModernGigabyte, ModernBill, and MBAPI are trademarks of ModernGigabyte, LLC. |
// +------------------------------------------------------------------------------+

require_once dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php";
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");

/**
 * This action displays a list of events, and the links to actions that act upon those events, to the screen. All items
 * displayed have already been added to the system queue. From this action, the user should be able to add, edit, or
 * remove items from the event queue. To get our list of events, we call GetSystemQueue.
 *
 * @author  Michael Fountain
 * @author  Vic Fryzel
 */
class ShowEvents extends MB_Action {
	/**
	 * The constructor for a ShowEvents action object. This merely calls the parent constructor,
	 * and sets the string representation of this object. This method also sets up $this->params and
	 * $this->sortColumns for use in $this->getMBAPIQuery().
	 */
	public function __construct() {
		parent::MB_Action();

		$this->params = array
        (
            "systemQueueID",
            "systemParentQueueID",
            "systemQueueCommand",
            "systemQueueCommandInput",
			"configGroupID",
			"serverGroupID",
			"systemQueueStatus",
			"systemQueueIsRecurring",
            "systemQueueDateCreated", "systemQueueDateCreatedLeast", "systemQueueDateCreatedGreatest",
            "systemQueueDateStarted", "systemQueueDateStartedLeast", "systemQueueDateStartedGreatest",
            "systemQueueDateFinished", "systemQueueDateFinishedLeast", "systemQueueDateFinishedGreatest"
        );

/*
		// if we are searching for any possible match, set VALID params now
		$validCompares = array("Least","Greatest");
		if ($_REQUEST["dateQuery"]) {
			// loop through each date entered
			foreach($_REQUEST["dateQuery"] as $key => $date) {
				// make sure the date is not empty
				if ($date && ($_REQUEST["dateCompare"][$key]
						|| in_array($_REQUEST["dateCompare"][$key], $validCompares))) {
					$_REQUEST[$_REQUEST["dateColumn"].$_REQUEST["dateCompare"][$key]] = $date;
				}
			}
		}
		if ($_REQUEST["query"]) {
			// test if the searching for a specific column
			if ($_REQUEST["queryColumn"]) {
				$_REQUEST[$_REQUEST["queryColumn"]] = $_REQUEST["query"];
			} else {
				// test if query is an int, if yes, assume we are searching for the ID
				if (((int) $_REQUEST["query"])) {
					$_REQUEST["systemQueueID"] = $_REQUEST["query"];
				} else {
					// if not an int, then search all other non-ID columns
					foreach($this->params as $value) {
						if ($value!="systemQueueID") {
							$_REQUEST[$value] = $_REQUEST["query"];
						}
					}
				}
			}
		}

		// add filtered columns to $this->params LAST, they are NOT searchable
		if (is_numeric($_REQUEST["systemQueueIsRecurring"])) {
			$this->params[] = "systemQueueIsRecurring";
		}
*/

		$this->sortColumns = array(
			"systemQueueID",
			"systemQueueDateToRun",
			"systemQueueCommand",
			"systemQueueCommandInput",
			"systemQueueDateCreated",
					"systemQueueDateStarted",
			"systemQueueDateFinished",
			"systemQueueIsRecurring"
			);
		// search in a specific column
		if (notNull($_REQUEST["queryColumn"])){
			if (notNull($_REQUEST["query"])) {
				$_REQUEST[$_REQUEST["queryColumn"]] = $_REQUEST["query"];
			}
		}
	}

	/**
	 * The main method of this class that executes the ShowEvents action, and returns success or failure.
	 *
	 * @return  int  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		parent::execute();

		$this->initListParams();
		$registrars = $this->getRegistrarOptions();
		$panels = $this->getPanelOptions();

		if(!(isset($_REQUEST['serverGroupID']) && $_REQUEST['serverGroupID'])) {
			unset($_REQUEST['serverGroupID']);
		}
		if(!(isset($_REQUEST['configGroupID']) && $_REQUEST['configGroupID'])) {
			unset($_REQUEST['configGroupID']);
		}
		if(!(isset($_REQUEST['systemQueueStatus']) && $_REQUEST['systemQueueStatus'] !== "")) {
			unset($_REQUEST['systemQueueStatus']);
		}
		if(!(isset($_REQUEST['systemQueueIsRecurring']) && $_REQUEST['systemQueueIsRecurring'] !== "")) {
			unset($_REQUEST['systemQueueIsRecurring']);
		}

		$c = count($_REQUEST["systemQueueIDs"]);
		if ($c > 0) {
			$xml = array("systemQueueIDs" => array("systemQueueID" => $_REQUEST["systemQueueIDs"]));
			$systemQueueIDsArgs = "";
			for($i=0;$i<$c;$i++){
				$systemQueueIDsArgs .= "&systemQueueIDs[]=".$_REQUEST["systemQueueIDs"][$i];
			}
		} else {
			$xml = array();
		}
		$query = $this->getMBAPIQuery("GetSystemQueue", "", $xml, 1, 1, 1, 1);
/*
show($_REQUEST);
echo '<pre>';
echo htmlentities($query);
echo '</pre>';
exit;
*/
		$data = dispatchMBAPI($query, 1);
		$results =& $data["mbapi"][0]["results"][0];
		if($this->addMBAPIErrors($data) === 0) {

			// set the sorting args for display
			$this->smartyVars["displaySortColumnName"] = $data["mbapi"][0]["header"][0]["sortColumnName"][0];
			$this->smartyVars["displaySortColumn"] = $data["mbapi"][0]["header"][0]["sortColumn"][0];
			$this->smartyVars["displaySortDir"] = $data["mbapi"][0]["header"][0]["sortDir"][0];

			$events = collapseArrayElements(protectArray($results["systemQueue"][0]["systemQueueEvent"]));
			if($events){
				foreach($events as $i => $event) {
					if(isset($events[$i]["systemQueueLogs"]["systemQueueLog"])) {
						$events[$i]["systemQueueLogs"]["systemQueueLog"] = (array) $events[$i]["systemQueueLogs"]["systemQueueLog"];
					}
				}
			}
			$this->smartyVars["events"] = $events;
			$this->smartyVars["registrars"] = $registrars;
			$this->smartyVars["panels"] = $panels;
			$this->smartyVars["statuses"] = getStatuses("queue");
			$this->smartyVars["paginationLinks"] = collapseArrayElements($results["paginationLinks"]);
			$this->smartyVars["sortColumns"] = array();
			$this->smartyVars["systemQueueIDsArgs"] = $systemQueueIDsArgs;


			$this->mergeServerGroupData();
			$this->addTemplate("events/ShowEvents");
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

	private function initListParams()
	{
		$listName = "UIEventList";
		$session = getCurrentSession();
		$params = Factory()->ListParamList(array(
			"user_id" => $session->user_id,
			"user_type" => $session->user_type,
			"list_name" => $listName,
		));
		$eventListParams = Factory()->ListParam(
			isset($params[0])
				? $params[0]->id
				: NULL
		);

		if (0 == $eventListParams->id)
		{
			// Default parameters
			$eventListParams->user_id = $session->user_id;
			$eventListParams->user_type = $session->user_type;
			$eventListParams->list_name = $listName;
			$eventListParams->sort = "systemQueueDateToRun";
			$eventListParams->reverse = 0;
			$eventListParams->page_num = 0;
			$eventListParams->update();
		}

		$sort = bgpc('sortColumn');
		if (!is_null($sort))
		{
			$sortDir = bgpc('sortDir');
			// if new command with the same sorting, then reverse sorting; else reset reverse
			$eventListParams->reverse = ("asc" == $sortDir) ? 0 : 1;
			$eventListParams->sort = $sort;
		}
		$_REQUEST["sortColumn"] = $eventListParams->sort;
		$_REQUEST["sortDir"] = (0 == $eventListParams->reverse) ? "asc" : "desc";

		$recordOffset = bgpc('recordOffset');
		$defaultNumRows = MBACTION_DEFAULT_NUM_RECORDS;
		if (!is_null($recordOffset)) {
			$eventListParams->page_num = $recordOffset/$defaultNumRows;
		}
		$_REQUEST["recordOffset"] = $eventListParams->page_num*$defaultNumRows;
		$eventListParams->update();
	}
} // ShowEvents
?>
