<?php

require_once dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php";
require_once mb_path("/lib-billing/include/accounting/Refund.php");

/**
 * This Action object takes information provided by the ProcessTransactionRefund action,
 * and uses it to refund the transaction amount to the customer.
 */
class ProcessTransactionRefundResponse extends MB_Action
{
	public function execute()
	{
		parent::execute();

		if ( is_null($transactionID = bgpc("transactionID")) ) {
			$this->addError(blmsg("TRANS_ACTIONERROR"), "No transaction ID given.");
		}
		if ( "" == ($key = bgpc("key")) ) {
			$this->addError(blmsg("TRANS_ACTIONERROR"), "No encryption key given.");
		}
		if ( $this->getNumErrors() > 0 ) {
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}

		$invoiceID = ((int) $_REQUEST["invoiceID"]);
		$markUnpaid = (int)$_REQUEST["markUnpaid"];

		if ($_REQUEST["refundType"]=="partial"){
			$_REQUEST["refundAmount"] = $this->getNumberFromInput($_REQUEST["lesserAmount"]);
			$markUnpaid = 0;
		}

		$results = dispatchMBAPI(toXML(array(
			"mbapi" => array(
				"command" => "GetEncryptedText",
				"params" => array(
					"text" => $key,
				),
			),
		)), 1);
		$key = ($results["mbapi"] && $results["mbapi"][0] && $results["mbapi"][0]["results"] && $results["mbapi"][0]["results"][0] && $results["mbapi"][0]["results"][0]["text"])?$results["mbapi"][0]["results"][0]["text"][0]:"";

        $gatewayAction = ($_REQUEST["gatewayAction"]=='void')?'void':'refund';

		$transaction = correctArray(grabFirstResultFromMBAPIArray(array(
			"mbapi" => array(
				"command" => "GetTransactions",
				"params" => array(
					"transactionIDs" => array(
						"transactionID" => $transactionID,
					),
				),
			),
		)));
		dbg($transaction);

		$billingAccount = oneByID(":billingAccounts", $transaction['billingAccountID']);
		if ( 0 == $billingAccount['billingAccountID'] ) {
			$this->addError(blmsg("TRANS_ACTIONERROR"), "Sorry, the billing account this refund is for was deleted.  To perform this refund please go to your merchant account provider.");
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}

		$refundAmount = doubleval($_REQUEST["refundAmount"])?doubleval($_REQUEST["refundAmount"]):round($transaction["transactionAmount"]-$transaction["transactionAmountRefunded"], 2);
		if ($refundAmount>round($transaction["transactionAmount"]-$transaction["transactionAmountRefunded"], 2)){
			$refundAmount = round($transaction["transactionAmount"]-$transaction["transactionAmountRefunded"], 2);
		}

		$q = new MBQuery("ProcessGateway", $gatewayAction,
			array(
				"paymentGatewayID" => $transaction["paymentGatewayID"],
				"transactionCode" => $transaction["transactionCode"],
				"billingAccountID" => $transaction["billingAccountID"],
				"paymentDescription" => TRANS_REFUND,
				"paymentAmount" => $refundAmount,
				"currencyID" => $transaction["currencyID"],
				"key" => $key,
			)
		);
		$r = $q->getResult();
		if ( !$r->getSuccess() ) {
			$this->addErrorsFromResult($r);
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}

		$s = $r->getSubjectFrom(array("mbapi", "results", "transactions", "transaction"));
		$refundReturns = $s->getTabular();
		$refundReturn = $refundReturns[0];
		if ( "successful" != $refundReturn["statusName"] ) {
			$this->addError(blmsg("TRANS_ACTIONERROR"), blmsg("TRANS_GATEWAY_RETURNED_ERROR") . ": " . $refundReturn["statusName"] . ": " . $refundReturn["message"]);
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}

		$invoiceInfo = correctArray(grabFirstResultFromMBAPIArray(array(
			"mbapi" => array(
				"command" => "GetInvoices",
				"params" => array(
					"invoiceID" => $invoiceID,
					"getInvoiceLineItemsData" => 1,
				),
			),
		)));

		$lineItemIDs = (array)$_REQUEST["lineItemIDs"];

		$transactionAmountRefunded = $transaction["transactionAmountRefunded"]+$refundAmount;

		if ($transaction["transactionAmount"]==$transactionAmountRefunded){
			$refundStatus = "refunded";
		} else {
			$refundStatus = "partially_refunded";
		}

		if (array_sum($lineItemIDs)){

			dbg(dispatchMBAPI(toXML(array(
				"mbapi" => array(
					"command" => "SetTransaction",
					"subCommand" => "update",
					"params" => array(
						"transactionID" => $transactionID,
						"authReturn" => getAPIStatusID("transactions", $refundStatus),
						"transactionAmountRefunded" => $transactionAmountRefunded,
					),
				),
			)), 1));

			if ($markUnpaid){
				dbg(dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetInvoices",
						"subCommand" => "update",
						"params" => array(
							"invoiceID" => $invoiceID,
							"invoiceStatus" => getAPIStatusID("invoice", "due"),
							"invoiceAmountPaid" => $invoiceInfo["invoiceAmount"]-$refundAmount,
						),
					),
				)), 1));

				foreach ((array)$lineItemIDs as $liID){
					dbg(dispatchMBAPI(toXML(array(
						"mbapi" => array(
							"command" => "SetTransactionSuccessRel",
							"subCommand" => "delete",
							"params" => array(
								"lineItemID" => $liID,
								"transactionID" => $transactionID,
							),
						),
					)), 1));
				}
			}

			$return = dispatchMBAPI(toXML(array(
				"mbapi" => array(
					"command" => "ProcessRefundAccounting",
					"params" => array(
						"transactionID" => $transactionID,
						"refundAmount" => $refundAmount,
						"ledgerAccountName" => ($markUnpaid)?"accounts_receivable":"refund_expense",
						"invoiceID" => $invoiceID,
					),
				),
			)), 1);
			if (!$return["mabpi"][0]["header"][0]["success"][0]){
				dbg($return);
			}

		} else {

			$clientID = $invoiceData["lineItems"]["lineItem"][0]["clientID"];

			if ($gatewayAction=="void"){
				$tStat = "voided";
			} else {
				$tStat = $refundStatus;
			}

			dbg(dispatchMBAPI(toXML(array(
				"mbapi" => array(
					"command" => "SetTransaction",
					"subCommand" => "update",
					"params" => array(
						"transactionID" => $transactionID,
						"authReturn" => getAPIStatusID("transactions", $tStat),
						"transactionAmountRefunded" => $transactionAmountRefunded,
					),
				),
			)), 1));

			if ($markUnpaid){
				dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetTransactionSuccessRel",
						"subCommand" => "delete",
						"params" => array(
							"transactionID" => $transactionID,
						),
					),
				)));
				$amountPaid = $invoiceInfo["invoiceAmountPaid"]-$transaction["transactionAmount"];
				$status = getAPIStatusID("invoice", "due");
				dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetInvoices",
						"subCommand" => "update",
						"params" => array(
							"invoiceID" => $invoiceID,
							"invoiceStatus" => $status,
							"invoiceAmountPaid" => $amountPaid,
						),
					),
				)));
			}

			$merge = array();
			if ($refundAmount!=$transaction["transactionAmount"]){
				$merge = array(
					"refundAmount" => $refundAmount,
				);
			}

			$return = dispatchMBAPI(toXML(array(
				"mbapi" => array(
					"command" => "ProcessRefundAccounting",
					"params" => array(
						"transactionID" => $transactionID,
						"ledgerAccountName" => ($markUnpaid)?"accounts_receivable":"refund_expense",
						"invoiceID" => $invoiceID,
					) + $merge,
				),
			)), 1);
			if (!$return["mabpi"][0]["header"][0]["success"][0]){
				dbg($return);
			}

		}

		if ($_REQUEST["redirectURL"]!="" && $_REQUEST["redirectAction"]!="") {
			redirect($_REQUEST["redirectURL"]."?action=".getActionID($_REQUEST["redirectAction"])."&".$_REQUEST["redirectArgs"]);
		} else {
			redirect("invoices.php?action=".getActionID("ShowInvoices")."&sid=".secureInput($_REQUEST["sid"]));
		}
	}
}