<?php
require_once("lib-action/action.php");
require_once("UI/Page.php");
require_once("objects/ControlPanel.php");

class EditPanelConfigPage extends UIPage
{
	public function createControls()
	{
		require_once("ModuleTool.php");
		require_once("DataObject.php");


		$this->module_input_controls = array();

		$controlPanel = Factory()->ControlPanel($this->getValue("configGroupID"));

		if (is_null($controlPanel->id))
		{
			$controlPanel->module_name = Factory()->ConfigGroup($this->getValue("parentConfigGroupID"))->config_group_name;
			$controlPanel->server_group_id = $this->getValue("serverGroupID");
		}

		foreach (ModuleTool::getModuleControls("panel", $controlPanel->module_name) as $inputPanel)
		{
			$inputPanel->initFromArray(DataObject::getMapOf($controlPanel->ConfigParams, "name", "value"));
			$this->module_input_controls = array_merge($this->module_input_controls, $inputPanel->getInputControls());
			if ( "" == $inputPanel->title ) {
				$inputPanel->title = blmsg("TRANS_CONFIGURATION_PARAMETERS");
			}
			$this->addInputPanel($inputPanel);
		}
		if (is_null(bgpc("command")) && ! is_null($controlPanel->id)) // Check Panel Connection here in view only mode
			$this->checkPanelConnection($controlPanel);
	}

	public function processInput()
	{
		$controlPanel = Factory()->ControlPanel($this->getValue("configGroupID"));
		$controlPanel->ConfigParams->clear();
		foreach ($this->module_input_controls as $control)
		{
			switch ($control->name)
			{
				case "modulePassword":
					require_once("lib-tk/include/crypt/crypt.php");
					if ("" != $control->value)
					{
						require_once("lib-tk/include/crypt/crypt.php");
						$controlPanel->ConfigParams[$control->name]->value = getEncryptedText(getStuff($control->value));
					}
					break;
				case "api":
					if ("" != $control->value)
						$controlPanel->ConfigParams[$control->name]->value = rtrim($control->value, "/");
					break;
				default:
					$controlPanel->ConfigParams[$control->name]->value = $control->value;
			}
		}

		if (! $this->checkPanelConnection($controlPanel, true))
			return false;

		try {
			require_once("object_managers/ControlPanelManager.php");
			ControlPanelManager::update($controlPanel);
			getCurrentSession()->Storage->infoMessages[] = blmsg("TRANS_PANEL_UPDATED_SUCCESSFULLY_MSG");
		} catch (Exception $e) {
			$this->addErrorMessage(UIFactory()->ErrorMessage($e->getMessage()));
		}

		return ! $this->hasWarnings() && ! $this->hasErrors();
	}

	private function checkPanelConnection(ControlPanel $controlPanel, $dont_show_version_warning = false)
	{
		if ($controlPanel->isPlesk()) {
			$error_message = NULL;
			try {
				require_once("PleskAPIFactory.php");
				PleskAPIFactory::getInstance(array("1.6.3.0"), $controlPanel);
			} catch (PleskAPIFactoryError $e) {
				if (PleskAPIFactoryError::PROTOCOL_NOT_FOUND == $e->getCode())
				{
					if (true !== $dont_show_version_warning)
						$this->addWarningMessage(UIFactory()->WarningMessage(blmsg("TRANS_PLESK_PANEL_DOES_NOT_SUPPORTED_LESS_VERSION_10_MSG")));
					return true;
				}
				$error_message = $e->getMessage();
			} catch (PleskAPIError $e) {
				$error_message = $e->getMessage();
			} catch (PleskAgentException $e) {
				$error_message = $e->getMessage();
			}
			if (! is_null($error_message))
			{
				$this->addErrorMessage(UIFactory()->ErrorMessage($error_message));
				return false;
			}
		}
		return true;
	}

	private $module_input_controls;
	private $plesk_sso_status;
}

class EditPanelConfig extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$config_group_id = bgpc("configGroupID");
		$parent_config_group_id = bgpc("parentConfigGroupID");
		$server_group_id = bgpc("serverGroupID");

		require_once("lib-tk/include/sso/SSOSettings.php");
		$controlPanel = Factory()->ControlPanel($config_group_id);
		if ($controlPanel->isPlesk() && SSOSettings::ssoIsEnabled()) {
			if (1 == bgpc("sync"))
			{
				$queue = Factory()->SystemQueue();
				$queue->config_group_id = $config_group_id;
				$queue->system_queue_command = "ProcessPanelsSync";
				$queue->system_queue_command_input =
					"<mbapi>
						<command>ProcessPanelsSync</command>
						<params>
							<panelID>{$config_group_id}</panelID>
						</params>
					</mbapi>";
				$queue->update();

				redirect(action_url("servergroups", "ShowServerGroups", array(
					"sync_started" => 1,
				)));
			}
		}

		$back_url = action_url("servergroups", "ShowServerGroups");
		$controller  = UIFactory()->PageController($back_url);
		$controller->addPage($edit_page = new EditPanelConfigPage($controller));
		$edit_page->title = blmsg("TRANS_EDIT_PANEL_OPTIONS");
		$edit_page->parent_title = blmsg("TRANS_MANAGE_SYSTEM");
		$edit_page->parent_url = $back_url;
		$controller->setValue("configGroupID", $config_group_id);
		$controller->setValue("parentConfigGroupID", $parent_config_group_id);
		$controller->setValue("serverGroupID", $server_group_id);
		$controller->processRequest();

		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}
