<?php
require_once(dirname(__FILE__)."/../../../../lib-action/action.php");
require_once("UI/Page.php");

class PaymentGatewayEditPage extends UIPage
{
	public function createControls()
	{
		$gateway_panel = UIFactory()->InputPanel(blmsg("TRANS_PAYMENT_GATEWAY"));
		$gateway = Factory()->PaymentGateway($this->getValue("payment_gateway_id"));
		$input_gateway_type = UIFactory()->InputLabel("payment_gateway_type", blmsg("TRANS_GATEWAY_TYPE"));
		require_once("object_managers/PaymentGatewayManager.php");
		$input_gateway_type->value = PaymentGatewayManager::typeToString($gateway->payment_gateway_type);
		$gateway_panel->addInputControl($input_gateway_type);
		$input_module = UIFactory()->InputLabel("payment_gateway_module", blmsg("TRANS_GATEWAY_MODULE"));
		$input_module->value = $gateway->moduleName;
		$gateway_panel->addInputControl($input_module);
		$input_gateway_name = UIFactory()->InputString("payment_gateway_name", blmsg("TRANS_GATEWAY_NAME"), true,
			array("PaymentGatewayManager", "checkName", $gateway->payment_gateway_id)
		);
		$input_gateway_name->value = $gateway->payment_gateway_name;
		$gateway_panel->addInputControl($input_gateway_name);
		$this->addInputPanel($gateway_panel);

		if ($this->isHSPCModuleSelected($gateway->payment_gateway_type))
		{
			$this->addHSPCModuleForm($gateway);
			return;
		}

		$this->module_input_controls = array();
		require_once("ModuleTool.php");
		require_once("DataObject.php");
		foreach ( ModuleTool::getModuleControls("gateway", $gateway->payment_gateway_module) as $control ) {
			$control->initFromArray(DataObject::getMapOf($gateway->PaymentGatewayParams, "name", "value"));
			$this->module_input_controls = array_merge($this->module_input_controls, $control->getInputControls());
			if ( "" == $control->title ) {
				$control->title = blmsg("TRANS_PAYMENT_GATEWAY_SETTINGS");
			}
			$this->addInputPanel($control);
		}
	}

	public function processInput()
	{
		$gateway = Factory()->PaymentGateway($this->getValue("payment_gateway_id"));
		$gateway->initFromArray($this->getValues(), null, array("payment_gateway_module", "payment_gateway_type"));
		$gateway->PaymentGatewayParams->clear();
		foreach ( $this->module_input_controls as $control ) {
			$gateway->PaymentGatewayParams[$control->name]->value = $control->value;
		}

		if ($this->isHSPCModuleSelected($gateway->payment_gateway_type))
		{
			if (! $this->processHSPCInput($gateway))
				return false;
		}

		require_once("object_managers/PaymentGatewayManager.php");
		PaymentGatewayManager::update($gateway);
		return true;
	}

	private function isHSPCModuleSelected($module_type)
	{
		require_once("objects/PaymentGateway.php");
		return PaymentGateway::TYPE_HSPC_CREDIT_CARD == $module_type || PaymentGateway::TYPE_HSPC_BANK_TRANSFER == $module_type;
	}

	private function addHSPCModuleForm(PaymentGateway $gateway)
	{
		require_once("UI/Panel/UIHSPCInputPanel.php");
		require_once("HSPC/Tools.php");
		require_once("ModuleTool.php");

		$form = ModuleTool::getModuleInstance("gateway", "HSPCGateway", array("gateway" => $gateway))
				->getPluginConfigForm();

		$this->getDefaultForm()->id = "form";
		$this->getDefaultForm()->name = "form";

		$this->addInputPanel(
			new UIHSPCInputPanel(blmsg("TRANS_HSPC_GATEWAY_SETTINGS"),
			HSPCTools::stylizePluginForm($form->form_content))
		);
	}

	private function processHSPCInput(PaymentGateway $gateway)
	{
		require_once("ModuleTool.php");
		$error = ModuleTool::getModuleInstance("gateway", "HSPCGateway", array("gateway" => $gateway))
				 ->collectPluginConfig();

		if ("" != $error)
		{
			$this->addErrorMessage(UIFactory()->ErrorMessage($error));
			return false;
		}
		return true;
	}

	private $module_input_controls;
}

class EditPaymentGateway extends MB_Action
{
	public function execute()
	{
		parent::execute();

		if ( is_null($payment_gateway_id = bgpc("payment_gateway_id")) ) {
             throw new Exception("Missing parameter 'payment_gateway_id'");
		}
		$back_url = action_url("paymentgateways", "ShowPaymentGateways");
		$controller  = UIFactory()->PageController($back_url);
		$controller->addPage($edit_page = new PaymentGatewayEditPage($controller));
		$edit_page->title = blmsg("TRANS_EDIT_PAYMENT_GATEWAY");
		$edit_page->parent_title = blmsg("TRANS_PAYMENT_GATEWAYS");
		$edit_page->parent_url = $back_url;
		$controller->setValue("payment_gateway_id", $payment_gateway_id);
		$controller->processRequest();

		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}