<?php

require_once("lib-action/action.php");
require_once("lib-mbapi/include/modules/module.php");
require_once("lib-tk/include/sso/SSOBillingFactory.php");
require_once("lib-tk/include/sso/SSOPleskAPIFactory.php");

class ShowServerGroups extends MB_Action
{
	public $params = array("serverGroupID", "serverGroupName");

	public $sortColumns = array("serverGroupID", "serverGroupName");

	public function execute()
	{
		parent::execute();

		$query = $this->getMBAPIQuery("GetServerGroups", "", array("getPanels" => 1));
		require_once("lib-mbapi/mbapi.php");
		$data = dispatchMBAPI($query, 1);

		$results =& $data["mbapi"][0]["results"][0];

		//Get the server names
		$query = $this->getMBAPIQuery("GetConfigs", "", array("configParamName" => "moduleDisplayName", "getFormAttributes" => 1));
		$data = dispatchMBAPI($query, 1);

		if($data["mbapi"][0]["header"][0]["numResults"][0]>0) {

			$serverNames = collapseArrayElements($data["mbapi"][0]["results"][0]);

			if(is_array($serverNames["configs"]["config"][0])) {
				foreach($serverNames["configs"]["config"] as $key => $value) {
					$temp[$value["configGroupID"]] = $value;
				}
				$serverNames = $temp;
				$temp = NULL;
			} else {
				$serverNames[$serverNames["configs"]["config"]["configGroupID"]] = $serverNames["configs"]["config"];
			}
		}

		//Get the max accounts for each panel
		$query = $this->getMBAPIQuery("GetConfigs", "", array("configParamName" => "panelMaxAccounts", "getFormAttributes" => 1));
		$data = dispatchMBAPI($query, 1);

		if($data["mbapi"][0]["header"][0]["numResults"][0]>0) {
			$maxAccounts = correctArray($data["mbapi"][0]["results"][0]["configs"][0]["config"]);
			foreach($maxAccounts as $value) {
				$serverNames[$value["configGroupID"]]["panelMaxAccounts"] = (int) $value["configParamValue"];
			}
		}

		//Get the current number of accounts per panel
		$query = $this->getMBAPIQuery("GetConfigs", "", array("configParamName" => "panelCurrentAccounts", "getFormAttributes" => 1));
		$data = dispatchMBAPI($query, 1);

		if($data["mbapi"][0]["header"][0]["numResults"][0]>0) {
			$maxAccounts = correctArray($data["mbapi"][0]["results"][0]["configs"][0]["config"]);
			foreach($maxAccounts as $value) {
				$serverNames[$value["configGroupID"]]["panelCurrentAccounts"] = (int) $value["configParamValue"];
			}
		}

		if($this->addMBAPIErrors($data) === 0) {
			// Get list of files in modules/panel to build panel list
			// Only if a Panel isn't associated yet, otherwise, must keep the same Panel.
			$panels = getAvailableModules("panel");
			sort($panels);
			foreach ($panels as $panelFile) {
				$panelFile = substr($panelFile, 0, -4);
				require_once mb_path("/lib-mbapi/include/modules/panel/".secureInput($panelFile).".php");
				$moduleInfo = call_user_func(array($panelFile,"getModuleInfo"));
				$p = new $panelFile(array("subCommand" => "getCapabilities"));;
				$canGetServerLoad = $p->capabilities[PANEL_GETSERVERLOAD];
				$useInternalIPQueue =  $p->capabilities[PANEL_USE_INTERNAL_IP_QUEUE];

				//The smarty var name panels will not work
				$this->smartyVars["controlPanels"][$panelFile] = $moduleInfo["displayName"];
				$this->smartyVars["canGetServerLoad"][$panelFile] = $canGetServerLoad;
				$this->smartyVars["useInternalIPQueue"][$panelFile] = $useInternalIPQueue;
				$this->smartyVars["canSyncWithServer"][$panelFile] = $p->capabilities[PANEL_SYNC];
			}

			$this->smartyVars["paginationLinks"] = $results["paginationLinks"][0];
			$this->smartyVars = collapseArrayElements(protectArray($this->smartyVars));
			$ips = array();
			// collapse the outer array first and properly format the sub array if 1 item or more than 1
			if(is_array($results["serverGroups"][0]["serverGroup"])) {
				foreach($results["serverGroups"][0]["serverGroup"] as $key => $value) {
					$panels = $value["panels"][0]["panel"];

					$details = correctArray($panels,"panels");

					foreach((array)$details as $panelInstance) {
						if($this->smartyVars["useInternalIPQueue"][strtolower($panelInstance["configGroupTransKey"])]) {
							//Pull the count of IPs for this panel instance
							$panelDetails = correctArray($panelInstance);

							$query = $this->getMBAPIQuery("GetIPLogs", "", array("configGroupID" => $panelDetails["configGroupID"]));
							$data = correctArray(dispatchMBAPI($query, 1));
							$ips[$panelDetails["configGroupID"]] = $data["mbapi"][0]["header"][0]["numTotalRecords"];
						}
					}
					$configGroups[$key] = collapseArrayElements($value);
					if(count($panels)>1) {
						$moduleName = Factory()->ControlPanel($panels[0]["configGroupID"][0])->module_name;
						$value = collapseArrayElements(arrayParentToChild($panels,"panels"));
						$configGroups[$key]["panels"] = $value;
						$configGroups[$key]["panelType"] = $moduleName;
						$configGroups[$key]["canGetServerLoad"] = $this->smartyVars["canGetServerLoad"][$moduleName];
						$configGroups[$key]["useInternalIPQueue"] = $this->smartyVars["useInternalIPQueue"][$moduleName];
						$configGroups[$key]["canSyncWithServer"] = $this->smartyVars["canSyncWithServer"][$moduleName];
					} else {
						$panels = collapseArrayElements($panels);
						unset($configGroups[$key]["panels"]);
						if($panels) {
							$moduleName = Factory()->ControlPanel($panels["configGroupID"])->module_name;
							$configGroups[$key]["panels"][0] = $panels;
							$configGroups[$key]["panelType"] = $moduleName;
							$configGroups[$key]["canGetServerLoad"] = $this->smartyVars["canGetServerLoad"][$moduleName];
							$configGroups[$key]["useInternalIPQueue"] = $this->smartyVars["useInternalIPQueue"][$moduleName];
							$configGroups[$key]["canSyncWithServer"] = $this->smartyVars["canSyncWithServer"][$moduleName];
						}
					}
				}
			}

			if(count($serverNames)>1) {
				$this->smartyVars["serverNames"] = $serverNames;
			} else {
				$this->smartyVars["serverNames"][0] = $serverNames;
			}
			if ($_REQUEST['sso_repairing_started']) {
				$this->smartyVars["sso_repairing_started"] = true;
			}
			$this->smartyVars["ips"] = $ips;
			$this->smartyVars["serverGroups"] = $configGroups;
			$this->mergeSystemNavData();
			$this->addTemplate("servers/ShowServerGroups", $this->smartyVars, 1);
		}
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}