<?php
require_once(dirname(__FILE__)."/../../../../lib-tk/shield.php");
require_once("lib-action/action.php");

class EditBillingAccount extends EditAction
{
	public function __construct()
	{
		global $CONF;
		$clientContactID = $CONF["user"]["clientContactID"];
		$clientID = (int)$CONF["user"]["id"];

		$contacts = arrayFor(":contacts", array(
			"clientID" => $clientID,
		));

		$keyedContacts = array();
		foreach ($contacts as $x=>$c){
			$keyedContacts[$c["clientContactID"]] =& $contacts[$x];
		}

		$this->smartyVars['isPrimary'] = $keyedContacts[$clientContactID]['contactPrimary'];

		$this->smartyVars['client'] = oneByID(":clients", $CONF['user']['id']);

		// if we are the primary contact and we can edit the billing accounts of
		// any contact for this client, else just edit our own
		if ((int)$keyedContacts[$clientContactID]["contactPrimary"] && is_array($keyedContacts[(int)$_REQUEST["clientContactID"]])){
			$clientContactID = (int)$_REQUEST["clientContactID"];
		}
		$_REQUEST["clientContactID"] = $clientContactID;

		$this->d = new DataSource("billingAccounts", array("clientContactID" => $clientContactID));
		$this->d->setSaltCallback("billingAccountSalter");
		$description = $this->d->getDescription();
		$description["names"]["file"] = "profile";
		$description["display"]["editTemplate"] = "profile/EditBillingAccount";
		$this->d->setDescription($description);
		parent::EditAction();
	}

	function editParamHandler()
	{
		global $CONF;
		$clientID = $CONF["user"]["id"];
		$clientContactID = $CONF["user"]["clientContactID"];
		$billingAccountID = (int) $_REQUEST["billingAccountID"];

		require_once("object_managers/SecurityManager.php");
		SecurityManager::pushCurrentActionIntoLog(
			SecurityManager::USER_TYPE_CLIENT,
			$clientID,
			Factory()->Client($clientID)->PrimaryContact->contact_username,
			"TRANS_ACTION_LOG_REASON_SHOW_BILLING_ACCOUNT_DETAILS",
			blmsg(
				"TRANS_ACTION_LOG_COMMENT_CLIENT_BILLING_ACCOUNT",
				array("ACCOUNT_NAME" => Factory()->ClientBillingAccount($billingAccountID)->getAccountNameForList(), "ACCOUNT_ID" => $billingAccountID, "CLIENT_ID" => $clientID),
				SecurityManager::getActionLogLocale()
			)
		);

		$contact = oneByID(":contacts", $clientContactID);
		$this->smartyVars['isPrimary'] = $contact['contactPrimary'];

		$this->smartyVars['client'] = oneByID(":clients", $CONF['user']['id']);

		loadConfigs("payment");

		$this->smartyVars["billingAccountTypes"] = getTypes("billingAccount");

		$this->smartyVars["bankAccountTypes"] = getTypes("bankAccount");
		$this->smartyVars["cardExtraTypes"] = getTypes("cardExtraIDs");

		$this->smartyVars["selectedPackages"] = array();
		$shares = arrayFor(":clientBillingAccountShares", array(
			"billingAccountID" => $billingAccountID,
			"package" => array(
				"column" => array(
					"packageID",
					"packageName",
				),
			),
		));

		$ptypes = $CONF["types"]["products"];
		unset($ptypes[getAPITypeID("products", "fee")]);

		$packages = arrayFor(":packages", array(
			"clientID" => $clientID,
			"packageType" => array_keys($ptypes),
			"column" => array(
				"packageID", "packageName"
			),
		));

		foreach ($shares as $share){
			$this->smartyVars["selectedPackages"][$share["packageID"]] = "(ID: ".$share["packageID"].") ".$share["package"][0]["packageName"];
		}


		$this->smartyVars["selectablePackages"] = array();
		foreach ($packages as $p){
			if (!isset($this->smartyVars["selectedPackages"][$p["packageID"]])){
				$this->smartyVars["selectablePackages"][$p["packageID"]] = "(ID: ".$p["packageID"].") ".$p["packageName"];
			}
		}

		$d = new DataSource(
			"gwTransactions",
			array(
				"billingAccountID" => $billingAccountID,
				"gwTransactionType" => "reject",
				"groupBy" => array(
					"column" => "paymentGatewayID",
				),
				"column" => "paymentGatewayID",
			)
		);
		$t =& $d->getTable();
		$gatewayIDs = $t->getColumnDistinct("paymentGatewayID");

		$rejects = array();
		foreach($gatewayIDs as $gatewayID) {
			$reject = oneOf(
				"gwTransactions",
				array(
					"billingAccountID" => $billingAccountID,
					"gwTransactionType" => "reject",
					"paymentGatewayID" => $gatewayID,
					"gwTransactionAttrs" => array(),
					"sortColumn" => "gwTransactionID",
					"sortDir" => "DESC",
					"numRecords" => 1,
				)
			);
			$t = new Table($reject["gwTransactionAttrs"], "gwTransactionAttrName", "gwTransactionAttrValue");
			$reject["attrs"] = $t->getOptions();
			$rejects[] = $reject;
		}
		$this->smartyVars["rejects"] = $rejects;

        require_once("object_managers/PaymentGatewayManager.php");
        $this->smartyVars["cardTypes"] = PaymentGatewayManager::getAcceptedCardTypes(Factory()->PaymentGatewayList());
		if (getAPITypeID("billingAccount", "hspcbanktransfer") == $this->smartyVars["billingAccount"]["billingAccountType"])
		{
			require_once("ModuleTool.php");
			$client_contact_id = NULL;

			$hspc_pp_plugin = ModuleTool::getModuleInstance("gateway", "HSPCBankTransfer", array(
				"gateway_id" => $this->smartyVars["billingAccount"]["paymentGatewayID"],
				"billing_account_id" => $this->smartyVars["billingAccount"]["billingAccountID"],
			));
			$form = $hspc_pp_plugin->getPaymentMethodForm("edit");
			$this->smartyVars["hspc_payment_method_form"] = $form["form_content"];
		}
		return true;
	}
}